<?php
/*
Some things to help Sunshine work with Yoast SEO plugin better
*/
/*
add_action( 'wp', 'sunshine_wpseo_disable_meta' );
function sunshine_wpseo_disable_meta( ) {
    if ( defined( 'WPSEO_PATH' ) ) {
        remove_filter( 'wp_head', array( 'SunshineFrontend', 'meta' ), 1 );
    }
}
add_filter( 'wp_title', 'sunshine_wpseo_title', 1000 );
add_filter( 'wpseo_opengraph_title', 'sunshine_wpseo_title', 1000 );
function sunshine_wpseo_title( $title ) {
    global $post;
    if ( ! defined( 'WPSEO_PATH' ) ) {
        return $title;
    }
    if ( !empty( SunshineFrontend::$current_image ) ) {
        //$title = 'This is a Sunshine image';
    } elseif ( !empty( SunshineFrontend::$current_gallery ) ) {
        $alt_post = $post;
        $post = SunshineFrontend::$current_gallery;
        $frontend = WPSEO_Frontend::get_instance();
        $frontend->reset();
        remove_filter( 'wpseo_title', 'sunshine_wpseo_title' );
        $title = $frontend->title( $title );
        $post = $alt_post;
    }
    return $title;
}

// Change the meta description in Yoast when viewing single Sunshine Gallery
add_filter( 'wpseo_metadesc', 'sunshine_wpseo_metadesc' );
function sunshine_wpseo_metadesc( $metadesc ) {
    if ( !empty( SunshineFrontend::$current_image ) ) {
        //$metadesc = 'This is a Sunshine image';
    } elseif ( !empty( SunshineFrontend::$current_gallery ) ) {
        $template = WPSEO_Options::get( 'metadesc-sunshine-gallery' );
        $metadesc_override = WPSEO_Meta::get_value( 'metadesc', SunshineFrontend::$current_gallery->ID );
        if ( is_string( $metadesc_override ) && '' !== $metadesc_override ) {
            $metadesc = $metadesc_override;
        } elseif ( '' !== $template ) {
            $metadesc  = $template;
        }
        $replacer = new WPSEO_Replace_Vars();
        $metadesc = $replacer->replace( $metadesc, SunshineFrontend::$current_gallery, array() );
    }
    return $metadesc;
}
*/

// Remove images from XML site map
//add_filter( 'wpseo_xml_sitemap_img', 'sunshine_wpseo_xml_sitemap_img', 99, 2 );
function sunshine_wpseo_xml_sitemap_img( $image, $post ) {
    global $sunshine;
    if ( $sunshine->options['hide_galleries'] ) {
        $parent_type = get_post_type( $post->post_parent );
        if ( 'sunshine-gallery' == $parent_type ) {
            $image = '';
        }
    }
    return $image;
}

add_filter( 'wpseo_sitemap_urlimages', 'sunshine_wpseo_sitemap_urlimages', 99, 2 );
function sunshine_wpseo_sitemap_urlimages( $images, $post_id ) {
    global $sunshine;
    if ( !$sunshine->options['hide_galleries'] && !post_password_required( $post_id ) ) {
        $parent_type = get_post_type( $post_id );
        if ( 'sunshine-gallery' == $parent_type ) {
            $attachments = get_attached_media( 'image', $post_id );
            foreach ( $attachments as $attachment ) {
                $image = wp_get_attachment_image_src( $attachment->ID, apply_filters( 'sunshine_image_size', 'full' ) );
                $images[] = array(
                    'src' => $image[0]
                );
            }
        }
    }
    return $images;
}

// Disallow all password protected and private gallery image folders
add_filter( 'robots_txt', 'sunshine_wpseo_robots_txt', 10, 2 );
function sunshine_wpseo_robots_txt( $output, $public ) {
    global $sunshine;

    $site_url = parse_url( site_url() );
	$path     = ( ! empty( $site_url['path'] ) ) ? $site_url['path'] : '';

    if ( $sunshine->options['hide_galleries'] ) { // Exclude all if this is the setting

        $output .= "User-agent: Googlebot-Image\n";
        $output .= "Disallow: $path/wp-content/uploads/sunshine/\n";

    } else {

        // Password protected galleries
        $args = array(
            'post_type' => 'sunshine-gallery',
            'has_password' => true,
            'nopaging' => true
        );
        $query = new WP_Query( $args );
        while ( $query->have_posts() ) : $query->the_post();
            $output .= "User-agent: Googlebot-Image\n";
            $output .= "Disallow: $path/wp-content/uploads/sunshine/" . get_the_ID() . "*.jpg$/\n";
        endwhile; wp_reset_postdata();

        // Private galleries
        $args = array(
            'post_type' => 'sunshine-gallery',
            'post_status' => 'private',
            'nopaging' => true
        );
        $query = new WP_Query( $args );
        while ( $query->have_posts() ) : $query->the_post();
            $output .= "User-agent: Googlebot-Image\n";
            $output .= "Disallow: $path/wp-content/uploads/sunshine/" . get_the_ID() . "/*.jpg$\n";
        endwhile; wp_reset_postdata();

    }

    return $output;
}


add_filter( 'jetpack_enable_open_graph', 'sunshine_seo_stop_if_gallery' );
add_filter( 'wpseo_opengraph_title', 'sunshine_seo_stop_if_gallery' );
add_filter( 'wpseo_opengraph_desc', 'sunshine_seo_stop_if_gallery' );
add_filter( 'wpseo_opengraph_type', 'sunshine_seo_stop_if_gallery' );
add_filter( 'wpseo_opengraph_site_name', 'sunshine_seo_stop_if_gallery' );
add_filter( 'wpseo_opengraph_url', 'sunshine_seo_stop_if_gallery' );
add_filter( 'wpseo_og_image', 'sunshine_seo_stop_if_gallery' );
add_filter( 'wpseo_canonical', 'sunshine_seo_stop_if_gallery' );
add_filter( 'wpseo_json_ld_output', 'sunshine_seo_stop_if_gallery' );

function sunshine_seo_stop_if_gallery( $value ) {
    if ( !empty( SunshineFrontend::$current_gallery ) ) {
        return false;
    }
    return $value;
}
